{
    "$graph": [
        {
            "class": "Workflow",
            "label": "Align and sort reads",
            "doc": "Align sorted bam with this subworkflow consisting of BWA Mem + SamTools + Gatk4SortSam",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                }
            ],
            "inputs": [
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sample_name",
                    "type": "string"
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/reference",
                    "type": "File",
                    "secondaryFiles": [
                        ".fai",
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        "^.dict"
                    ]
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/fastq",
                    "type": {
                        "type": "array",
                        "items": "File"
                    }
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_adapter",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ]
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_removeMiddle3Adapter",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ]
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_front",
                    "doc": "(-g)  Sequence of an adapter ligated to the 5' end (paired data: of the first read). The adapter and any preceding bases are trimmed. Partial matches at the 5' end are allowed. If a '^' character is prepended ('anchoring'), the adapter is only found if it is a prefix of the read.",
                    "type": [
                        "string",
                        "null"
                    ]
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_removeMiddle5Adapter",
                    "doc": "5' adapter to be removed from second read in a pair.",
                    "type": [
                        "string",
                        "null"
                    ]
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_qualityCutoff",
                    "doc": "(]3'CUTOFF, ]3'CUTOFF, -q)  Trim low-quality bases from 5' and/or 3' ends of each read before adapter removal. Applied to both reads if data is paired. If one value is given, only the 3' end is trimmed. If two comma-separated cutoffs are given, the 5' end is trimmed with the first cutoff, the 3' end with the second.",
                    "type": "int",
                    "default": 15
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_minimumLength",
                    "doc": "(-m)  Discard reads shorter than LEN. Default: 0",
                    "type": "int",
                    "default": 50
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/bwamem_markShorterSplits",
                    "doc": "Mark shorter split hits as secondary (for Picard compatibility).",
                    "type": "boolean",
                    "default": true
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_sortOrder",
                    "doc": "The --SORT_ORDER argument is an enumerated type (SortOrder), which can have one of the following values: [unsorted, queryname, coordinate, duplicate, unknown]",
                    "type": "string",
                    "default": "coordinate"
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_createIndex",
                    "doc": "Whether to create a BAM index when writing a coordinate-sorted BAM file.",
                    "type": "boolean",
                    "default": true
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_validationStringency",
                    "doc": "Validation stringency for all SAM files read by this program. Setting stringency to SILENT can improve performance when processing a BAM file in which variable-length data (read, qualities, tags) do not otherwise need to be decoded.The --VALIDATION_STRINGENCY argument is an enumerated type (ValidationStringency), which can have one of the following values: [STRICT, LENIENT, SILENT]",
                    "type": "string",
                    "default": "SILENT"
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_maxRecordsInRam",
                    "doc": "When writing SAM files that need to be sorted, this will specify the number of records stored in RAM before spilling to disk. Increasing this number reduces the number of file handles needed to sort a SAM file, and increases the amount of RAM needed.",
                    "type": "int",
                    "default": 5000000
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_tmpDir",
                    "doc": "Undocumented option",
                    "type": "string",
                    "default": "."
                }
            ],
            "steps": [
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt",
                    "label": "Cutadapt",
                    "in": [
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt/fastq",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/fastq"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt/adapter",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_adapter"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt/front",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_front"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt/qualityCutoff",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_qualityCutoff"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt/minimumLength",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_minimumLength"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt/removeMiddle3Adapter",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_removeMiddle3Adapter"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt/removeMiddle5Adapter",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt_removeMiddle5Adapter"
                        }
                    ],
                    "run": "#cutadapt_2_1.cwl",
                    "out": [
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt/out"
                        }
                    ]
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/bwamem",
                    "label": "Bwa mem + Samtools View",
                    "in": [
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/bwamem/reference",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/reference"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/bwamem/reads",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/cutadapt/out"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/bwamem/sampleName",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/sample_name"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/bwamem/markShorterSplits",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/bwamem_markShorterSplits"
                        }
                    ],
                    "run": "#BwaMemSamtoolsView_0_7_17_1_9.cwl",
                    "out": [
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/bwamem/out"
                        }
                    ]
                },
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam",
                    "label": "GATK4: SortSAM",
                    "in": [
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam/bam",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/bwamem/out"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam/sortOrder",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_sortOrder"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam/createIndex",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_createIndex"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam/maxRecordsInRam",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_maxRecordsInRam"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam/tmpDir",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_tmpDir"
                        },
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam/validationStringency",
                            "source": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam_validationStringency"
                        }
                    ],
                    "run": "#Gatk4SortSam_4_1_2_0.cwl",
                    "out": [
                        {
                            "id": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam/out"
                        }
                    ]
                }
            ],
            "id": "#BwaAligner_1_0_0.cwl",
            "outputs": [
                {
                    "id": "#BwaAligner_1_0_0.cwl/BwaAligner/out",
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "outputSource": "#BwaAligner_1_0_0.cwl/BwaAligner/sortsam/out"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "label": "Bwa mem + Samtools View",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "michaelfranklin/bwasamtools:0.7.17-1.9"
                }
            ],
            "inputs": [
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/reference",
                    "label": "reference",
                    "type": "File",
                    "secondaryFiles": [
                        ".fai",
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        "^.dict"
                    ],
                    "inputBinding": {
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/reads",
                    "label": "reads",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 3,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/mates",
                    "label": "mates",
                    "type": [
                        {
                            "type": "array",
                            "items": "File"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "position": 4,
                        "itemSeparator": " ",
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/outputFilename",
                    "label": "outputFilename",
                    "doc": "output file name [stdout]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.bam",
                    "inputBinding": {
                        "prefix": "-o",
                        "position": 8,
                        "valueFrom": "$(inputs.sampleName).bam",
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/sampleName",
                    "label": "sampleName",
                    "doc": "Used to construct the readGroupHeaderLine with format: '@RG\\tID:{name}\\tSM:{name}\\tLB:{name}\\tPL:ILLUMINA'",
                    "type": "string"
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/platformTechnology",
                    "label": "platformTechnology",
                    "doc": "(ReadGroup: PL) Used to construct the readGroupHeaderLine, defaults: ILLUMINA",
                    "type": "string",
                    "default": "ILLUMINA"
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/minimumSeedLength",
                    "label": "minimumSeedLength",
                    "doc": "Matches shorter than INT will be missed. The alignment speed is usually insensitive to this value unless it significantly deviates 20. (Default: 19)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-k",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/batchSize",
                    "label": "batchSize",
                    "doc": "Process INT input bases in each batch regardless of the number of threads in use [10000000*nThreads]. By default, the batch size is proportional to the number of threads in use. Because the inferred insert size distribution slightly depends on the batch size, using different number of threads may produce different output. Specifying this option helps reproducibility.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-K",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/useSoftClippingForSupplementaryAlignments",
                    "label": "useSoftClippingForSupplementaryAlignments",
                    "doc": "Use soft clipping CIGAR operation for supplementary alignments. By default, BWA-MEM uses soft clipping for the primary alignment and hard clipping for supplementary alignments.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-Y",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/bandwidth",
                    "label": "bandwidth",
                    "doc": "Essentially, gaps longer than ${bandWidth} will not be found. Note that the maximum gap length is also affected by the scoring matrix and the hit length, not solely determined by this option. (Default: 100)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-w",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/offDiagonalXDropoff",
                    "label": "offDiagonalXDropoff",
                    "doc": "(Z-dropoff): Stop extension when the difference between the best and the current extension score is above |i-j|*A+INT, where i and j are the current positions of the query and reference, respectively, and A is the matching score. Z-dropoff is similar to BLAST\u2019s X-dropoff except that it doesn\u2019t penalize gaps in one of the sequences in the alignment. Z-dropoff not only avoids unnecessary extension, but also reduces poor alignments inside a long good alignment. (Default: 100)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-d",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/reseedTrigger",
                    "label": "reseedTrigger",
                    "doc": "Trigger re-seeding for a MEM longer than minSeedLen*FLOAT. This is a key heuristic parameter for tuning the performance. Larger value yields fewer seeds, which leads to faster alignment speed but lower accuracy. (Default: 1.5)",
                    "type": [
                        "float",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-r",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/occurenceDiscard",
                    "label": "occurenceDiscard",
                    "doc": "Discard a MEM if it has more than INT occurence in the genome. This is an insensitive parameter. (Default: 10000)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-c",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/performSW",
                    "label": "performSW",
                    "doc": "In the paired-end mode, perform SW to rescue missing hits only but do not try to find hits that fit a proper pair.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-P",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/matchingScore",
                    "label": "matchingScore",
                    "doc": "Matching score. (Default: 1)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-A",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/mismatchPenalty",
                    "label": "mismatchPenalty",
                    "doc": "Mismatch penalty. The sequence error rate is approximately: {.75 * exp[-log(4) * B/A]}. (Default: 4)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-B",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/openGapPenalty",
                    "label": "openGapPenalty",
                    "doc": "Gap open penalty. (Default: 6)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-O",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/gapExtensionPenalty",
                    "label": "gapExtensionPenalty",
                    "doc": "Gap extension penalty. A gap of length k costs O + k*E (i.e. -O is for opening a zero-length gap). (Default: 1)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-E",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/clippingPenalty",
                    "label": "clippingPenalty",
                    "doc": "Clipping penalty. When performing SW extension, BWA-MEM keeps track of the best score reaching the end of query. If this score is larger than the best SW score minus the clipping penalty, clipping will not be applied. Note that in this case, the SAM AS tag reports the best SW score; clipping penalty is not deducted. (Default: 5)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-L",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/unpairedReadPenalty",
                    "label": "unpairedReadPenalty",
                    "doc": "Penalty for an unpaired read pair. BWA-MEM scores an unpaired read pair as scoreRead1+scoreRead2-INT and scores a paired as scoreRead1+scoreRead2-insertPenalty. It compares these two scores to determine whether we should force pairing. (Default: 9)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-U",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/assumeInterleavedFirstInput",
                    "label": "assumeInterleavedFirstInput",
                    "doc": "Assume the first input query file is interleaved paired-end FASTA/Q. ",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-p",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/outputAlignmentThreshold",
                    "label": "outputAlignmentThreshold",
                    "doc": "Don\u2019t output alignment with score lower than INT. Only affects output. (Default: 30)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-T",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/outputAllElements",
                    "label": "outputAllElements",
                    "doc": "Output all found alignments for single-end or unpaired paired-end reads. These alignments will be flagged as secondary alignments.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-a",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/appendComments",
                    "label": "appendComments",
                    "doc": "Append append FASTA/Q comment to SAM output. This option can be used to transfer read meta information (e.g. barcode) to the SAM output. Note that the FASTA/Q comment (the string after a space in the header line) must conform the SAM spec (e.g. BC:Z:CGTAC). Malformated comments lead to incorrect SAM output.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-C",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/hardClipping",
                    "label": "hardClipping",
                    "doc": "Use hard clipping \u2019H\u2019 in the SAM output. This option may dramatically reduce the redundancy of output when mapping long contig or BAC sequences.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-H",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/markShorterSplits",
                    "label": "markShorterSplits",
                    "doc": "Mark shorter split hits as secondary (for Picard compatibility).",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-M",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/verboseLevel",
                    "label": "verboseLevel",
                    "doc": "Control the verbose level of the output. This option has not been fully supported throughout BWA. Ideally, a value: 0 for disabling all the output to stderr; 1 for outputting errors only; 2 for warnings and errors; 3 for all normal messages; 4 or higher for debugging. When this option takes value 4, the output is not SAM. (Default: 3)",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-v",
                        "position": 2,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/skippedReadsOutputFilename",
                    "label": "skippedReadsOutputFilename",
                    "doc": "output reads not selected by filters to FILE [null]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-U",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/referenceIndex",
                    "label": "referenceIndex",
                    "doc": "FILE listing reference names and lengths (see long help) [null]",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-t",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/intervals",
                    "label": "intervals",
                    "doc": "only include reads overlapping this BED FILE [null]",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-L",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/includeReadsInReadGroup",
                    "label": "includeReadsInReadGroup",
                    "doc": "only include reads in read group STR [null]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-r",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/includeReadsInFile",
                    "label": "includeReadsInFile",
                    "doc": "only include reads with read group listed in FILE [null]",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-R",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/includeReadsWithQuality",
                    "label": "includeReadsWithQuality",
                    "doc": "only include reads with mapping quality >= INT [0]",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-q",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/includeReadsInLibrary",
                    "label": "includeReadsInLibrary",
                    "doc": "only include reads in library STR [null]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-l",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/includeReadsWithCIGAROps",
                    "label": "includeReadsWithCIGAROps",
                    "doc": "only include reads with number of CIGAR operations consuming query sequence >= INT [0]",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-m",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/includeReadsWithAllFLAGs",
                    "label": "includeReadsWithAllFLAGs",
                    "doc": "only include reads with all of the FLAGs in INT present [0]",
                    "type": [
                        {
                            "type": "array",
                            "items": "int"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-f",
                        "position": 8,
                        "itemSeparator": " ",
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/includeReadsWithoutFLAGs",
                    "label": "includeReadsWithoutFLAGs",
                    "doc": "only include reads with none of the FLAGS in INT present [0]",
                    "type": [
                        {
                            "type": "array",
                            "items": "int"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-F",
                        "position": 8,
                        "itemSeparator": " ",
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/excludeReadsWithAllFLAGs",
                    "label": "excludeReadsWithAllFLAGs",
                    "doc": "only EXCLUDE reads with all of the FLAGs in INT present [0] fraction of templates/read pairs to keep; INT part sets seed)",
                    "type": [
                        {
                            "type": "array",
                            "items": "int"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-G",
                        "position": 8,
                        "itemSeparator": " ",
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/useMultiRegionIterator",
                    "label": "useMultiRegionIterator",
                    "doc": "use the multi-region iterator (increases the speed, removes duplicates and outputs the reads as they are ordered in the file)",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-M",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/readTagToStrip",
                    "label": "readTagToStrip",
                    "doc": "read tag to strip (repeatable) [null]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-x",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/collapseBackwardCIGAROps",
                    "label": "collapseBackwardCIGAROps",
                    "doc": "collapse the backward CIGAR operation Specify a single input file format option in the form of OPTION or OPTION=VALUE",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-B",
                        "position": 8,
                        "shellQuote": false
                    }
                },
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/outputFmt",
                    "label": "outputFmt",
                    "doc": "(OPT[, -O)  Specify output format (SAM, BAM, CRAM) Specify a single output file format option in the form of OPTION or OPTION=VALUE",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--output-fmt",
                        "position": 8,
                        "shellQuote": false
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl/BwaMemSamtoolsView/out",
                    "label": "out",
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.sampleName).bam",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "arguments": [
                {
                    "position": 0,
                    "valueFrom": "bwa",
                    "shellQuote": false
                },
                {
                    "position": 1,
                    "valueFrom": "mem",
                    "shellQuote": false
                },
                {
                    "position": 5,
                    "valueFrom": "|",
                    "shellQuote": false
                },
                {
                    "position": 6,
                    "valueFrom": "samtools",
                    "shellQuote": false
                },
                {
                    "position": 7,
                    "valueFrom": "view",
                    "shellQuote": false
                },
                {
                    "prefix": "-T",
                    "position": 8,
                    "valueFrom": "$(inputs.reference)",
                    "shellQuote": false
                },
                {
                    "prefix": "--threads",
                    "position": 8,
                    "valueFrom": "$([inputs.runtime_cpu, 16, 1].filter(function (inner) { return inner != null })[0])",
                    "shellQuote": false
                },
                {
                    "position": 8,
                    "valueFrom": "-h",
                    "shellQuote": false
                },
                {
                    "position": 8,
                    "valueFrom": "-b",
                    "shellQuote": false
                },
                {
                    "prefix": "-R",
                    "position": 2,
                    "valueFrom": "$(\"@RG\\\\tID:{name}\\\\tSM:{name}\\\\tLB:{name}\\\\tPL:{pl}\".replace(/\\{name\\}/g, inputs.sampleName).replace(/\\{pl\\}/g, inputs.platformTechnology))"
                },
                {
                    "prefix": "-t",
                    "position": 2,
                    "valueFrom": "$([inputs.runtime_cpu, 16, 1].filter(function (inner) { return inner != null })[0])",
                    "shellQuote": false
                }
            ],
            "id": "#BwaMemSamtoolsView_0_7_17_1_9.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "GATK4: CollectInsertSizeMetrics",
            "doc": "Provides useful metrics for validating library construction including the insert size distribution and read orientation of paired-end libraries",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.1.3.0"
                }
            ],
            "inputs": [
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/javaOptions",
                    "label": "javaOptions",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/compression_level",
                    "label": "compression_level",
                    "doc": "Compression level for all compressed files created (e.g. BAM and VCF). Default value: 2.",
                    "type": [
                        "int",
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/bam",
                    "label": "bam",
                    "doc": "Input SAM or BAM file.  Required.",
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "inputBinding": {
                        "prefix": "-I",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/outputFilename",
                    "label": "outputFilename",
                    "doc": "File to write the output to.  Required.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.metrics.txt",
                    "inputBinding": {
                        "prefix": "-O"
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/outputHistogram",
                    "label": "outputHistogram",
                    "doc": "File to write insert size Histogram chart to.  Required. ",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.histogram.pdf",
                    "inputBinding": {
                        "prefix": "-H"
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/argumentsFile",
                    "label": "argumentsFile",
                    "doc": "read one or more arguments files and add them to the command line",
                    "type": [
                        {
                            "type": "array",
                            "inputBinding": {
                                "prefix": "--arguments_file"
                            },
                            "items": "File"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/assumeSorted",
                    "label": "assumeSorted",
                    "doc": "If true (default), then the sort order in the header file will be ignored.  Default value: true. Possible values: {true, false}",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--ASSUME_SORTED",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/deviations",
                    "label": "deviations",
                    "doc": "Generate mean, sd and plots by trimming the data down to MEDIAN + DEVIATIONS*MEDIAN_ABSOLUTE_DEVIATION. This is done because insert size data typically includes enough anomalous values from chimeras and other artifacts to make the mean and sd grossly misleading regarding the real distribution.  Default value: 10.0. ",
                    "type": [
                        "double",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--DEVIATIONS",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/histogramWidth",
                    "label": "histogramWidth",
                    "doc": "Explicitly sets the Histogram width, overriding automatic truncation of Histogram tail. Also, when calculating mean and standard deviation, only bins <= Histogram_WIDTH will be included.  Default value: null. ",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--HISTOGRAM_WIDTH",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/includeDuplicates",
                    "label": "includeDuplicates",
                    "doc": "If true, also include reads marked as duplicates in the insert size histogram.  Default value: false. Possible values: {true, false} ",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--INCLUDE_DUPLICATES",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/metricAccumulationLevel",
                    "label": "metricAccumulationLevel",
                    "doc": "The level(s) at  which to accumulate metrics.    This argument may be specified 0 or more times. Default value: [ALL_READS]. Possible values: {ALL_READS, SAMPLE, LIBRARY, READ_GROUP} .",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--METRIC_ACCUMULATION_LEVEL",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/minimumPCT",
                    "label": "minimumPCT",
                    "doc": "When generating the Histogram, discard any data categories (out of FR, TANDEM, RF) that have fewer than this percentage of overall reads. (Range: 0 to 1).  Default value: 0.05.",
                    "type": [
                        "float",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--MINIMUM_PCT",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/stopAfter",
                    "label": "stopAfter",
                    "doc": "Stop after  processing N reads, mainly for debugging.  Default value: 0. ",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--STOP_AFTER",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/version",
                    "label": "version",
                    "doc": "display the version number for this tool Default value: false. Possible values: {true, false}",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--version",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/showHidden",
                    "label": "showHidden",
                    "doc": "display hidden  arguments  Default  value: false.  Possible values: {true, false} ",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--showHidden",
                        "position": 11
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/out",
                    "label": "out",
                    "type": "File",
                    "outputBinding": {
                        "glob": "generated.metrics.txt",
                        "loadContents": false
                    }
                },
                {
                    "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl/Gatk4CollectInsertSizeMetrics/outHistogram",
                    "label": "outHistogram",
                    "type": "File",
                    "outputBinding": {
                        "glob": "generated.histogram.pdf",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": [
                "gatk",
                "CollectInsertSizeMetrics"
            ],
            "arguments": [
                {
                    "prefix": "--java-options",
                    "position": -1,
                    "valueFrom": "$(\"-Xmx{memory}G {compression} {otherargs}\".replace(/\\{memory\\}/g, (([inputs.runtime_memory, 8, 4].filter(function (inner) { return inner != null })[0] * 3) / 4)).replace(/\\{compression\\}/g, (inputs.compression_level != null) ? (\"-Dsamjdk.compress_level=\" + inputs.compression_level) : \"\").replace(/\\{otherargs\\}/g, [inputs.javaOptions, []].filter(function (inner) { return inner != null })[0].join(\" \")))"
                }
            ],
            "id": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "GATK4: Mark Duplicates",
            "doc": "MarkDuplicates (Picard): Identifies duplicate reads.\n\nThis tool locates and tags duplicate reads in a BAM or SAM file, where duplicate reads are \ndefined as originating from a single fragment of DNA. Duplicates can arise during sample \npreparation e.g. library construction using PCR. See also EstimateLibraryComplexity for \nadditional notes on PCR duplication artifacts. Duplicate reads can also result from a single \namplification cluster, incorrectly detected as multiple clusters by the optical sensor of the \nsequencing instrument. These duplication artifacts are referred to as optical duplicates.\n\nThe MarkDuplicates tool works by comparing sequences in the 5 prime positions of both reads \nand read-pairs in a SAM/BAM file. An BARCODE_TAG option is available to facilitate duplicate\nmarking using molecular barcodes. After duplicate reads are collected, the tool differentiates \nthe primary and duplicate reads using an algorithm that ranks reads by the sums of their \nbase-quality scores (default method).\n\nThe tool's main output is a new SAM or BAM file, in which duplicates have been identified \nin the SAM flags field for each read. Duplicates are marked with the hexadecimal value of 0x0400, \nwhich corresponds to a decimal value of 1024. If you are not familiar with this type of annotation, \nplease see the following blog post for additional information.\n\nAlthough the bitwise flag annotation indicates whether a read was marked as a duplicate, \nit does not identify the type of duplicate. To do this, a new tag called the duplicate type (DT) \ntag was recently added as an optional output in the 'optional field' section of a SAM/BAM file. \nInvoking the TAGGING_POLICY option, you can instruct the program to mark all the duplicates (All), \nonly the optical duplicates (OpticalOnly), or no duplicates (DontTag). The records within the \noutput of a SAM/BAM file will have values for the 'DT' tag (depending on the invoked TAGGING_POLICY), \nas either library/PCR-generated duplicates (LB), or sequencing-platform artifact duplicates (SQ). \nThis tool uses the READ_NAME_REGEX and the OPTICAL_DUPLICATE_PIXEL_DISTANCE options as the \nprimary methods to identify and differentiate duplicate types. Set READ_NAME_REGEX to null to \nskip optical duplicate detection, e.g. for RNA-seq or other data where duplicate sets are \nextremely large and estimating library complexity is not an aim. Note that without optical \nduplicate counts, library size estimation will be inaccurate.\n\nMarkDuplicates also produces a metrics file indicating the numbers \nof duplicates for both single- and paired-end reads.\n\nThe program can take either coordinate-sorted or query-sorted inputs, however the behavior \nis slightly different. When the input is coordinate-sorted, unmapped mates of mapped records \nand supplementary/secondary alignments are not marked as duplicates. However, when the input \nis query-sorted (actually query-grouped), then unmapped mates and secondary/supplementary \nreads are not excluded from the duplication test and can be marked as duplicate reads.\n\nIf desired, duplicates can be removed using the REMOVE_DUPLICATE and REMOVE_SEQUENCING_DUPLICATES options.",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.1.3.0"
                }
            ],
            "inputs": [
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/bam",
                    "label": "bam",
                    "doc": "One or more input SAM or BAM files to analyze. Must be coordinate sorted.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "prefix": "-I",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/outputFilename",
                    "label": "outputFilename",
                    "doc": "File to write duplication metrics to",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.markduped.bam",
                    "inputBinding": {
                        "prefix": "-O",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/metricsFilename",
                    "label": "metricsFilename",
                    "doc": "The output file to write marked records to.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.metrics.txt",
                    "inputBinding": {
                        "prefix": "-M",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/javaOptions",
                    "label": "javaOptions",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/compression_level",
                    "label": "compression_level",
                    "doc": "Compression level for all compressed files created (e.g. BAM and VCF). Default value: 2.",
                    "type": [
                        "int",
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/argumentsFile",
                    "label": "argumentsFile",
                    "doc": "read one or more arguments files and add them to the command line",
                    "type": [
                        {
                            "type": "array",
                            "items": "File"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--arguments_file",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/assumeSortOrder",
                    "label": "assumeSortOrder",
                    "doc": "If not null, assume that the input file has this order even if the header says otherwise. Exclusion: This argument cannot be used at the same time as ASSUME_SORTED. The --ASSUME_SORT_ORDER argument is an enumerated type (SortOrder), which can have one of the following values: [unsorted, queryname, coordinate, duplicate, unknown]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-ASO"
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/barcodeTag",
                    "label": "barcodeTag",
                    "doc": "Barcode SAM tag (ex. BC for 10X Genomics)",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--BARCODE_TAG"
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/comment",
                    "label": "comment",
                    "doc": "Comment(s) to include in the output file's header.",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-CO"
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/createIndex",
                    "label": "createIndex",
                    "doc": "Whether to create a BAM index when writing a coordinate-sorted BAM file.",
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "prefix": "--CREATE_INDEX",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/createMd5File",
                    "label": "createMd5File",
                    "doc": "Whether to create an MD5 digest for any BAM or FASTQ files created.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--CREATE_MD5_FILE",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/maxRecordsInRam",
                    "label": "maxRecordsInRam",
                    "doc": "When writing SAM files that need to be sorted, this will specify the number of records stored in RAM before spilling to disk. Increasing this number reduces the number of file handles needed to sort a SAM file, and increases the amount of RAM needed.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--MAX_RECORDS_IN_RAM",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/quiet",
                    "label": "quiet",
                    "doc": "Whether to suppress job-summary info on System.err.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--QUIET",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/tmpDir",
                    "label": "tmpDir",
                    "doc": "Undocumented option",
                    "type": "string",
                    "default": "tmp/",
                    "inputBinding": {
                        "prefix": "--TMP_DIR",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/useJdkDeflater",
                    "label": "useJdkDeflater",
                    "doc": "Whether to use the JdkDeflater (as opposed to IntelDeflater)",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--use_jdk_deflater",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/useJdkInflater",
                    "label": "useJdkInflater",
                    "doc": "Whether to use the JdkInflater (as opposed to IntelInflater)",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--use_jdk_inflater",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/validationStringency",
                    "label": "validationStringency",
                    "doc": "Validation stringency for all SAM files read by this program. Setting stringency to SILENT can improve performance when processing a BAM file in which variable-length data (read, qualities, tags) do not otherwise need to be decoded.The --VALIDATION_STRINGENCY argument is an enumerated type (ValidationStringency), which can have one of the following values: [STRICT, LENIENT, SILENT]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--VALIDATION_STRINGENCY",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/verbosity",
                    "label": "verbosity",
                    "doc": "The --verbosity argument is an enumerated type (LogLevel), which can have one of the following values: [ERROR, WARNING, INFO, DEBUG]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--verbosity",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/opticalDuplicatePixelDistance",
                    "label": "opticalDuplicatePixelDistance",
                    "doc": "The maximum offset between two duplicate clusters in order to consider them optical duplicates. The default is appropriate for unpatterned versions of the Illumina platform. For the patterned flowcell models, 2500 is more appropriate. For other platforms and models, users should experiment to find what works best.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--OPTICAL_DUPLICATE_PIXEL_DISTANCE"
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/out",
                    "label": "out",
                    "type": "File",
                    "secondaryFiles": [
                        "${\n\n        function resolveSecondary(base, secPattern) {\n          if (secPattern[0] == \"^\") {\n            var spl = base.split(\".\");\n            var endIndex = spl.length > 1 ? spl.length - 1 : 1;\n            return resolveSecondary(spl.slice(undefined, endIndex).join(\".\"), secPattern.slice(1));\n          }\n          return base + secPattern\n        }\n        return [\n                {\n                    path: resolveSecondary(self.path, \"^.bai\"),\n                    basename: resolveSecondary(self.basename, \".bai\"),\n                    class: \"File\",\n                }\n        ];\n\n}"
                    ],
                    "outputBinding": {
                        "glob": "generated.markduped.bam",
                        "loadContents": false
                    }
                },
                {
                    "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl/Gatk4MarkDuplicates/metrics",
                    "label": "metrics",
                    "type": "File",
                    "outputBinding": {
                        "glob": "generated.metrics.txt",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": [
                "gatk",
                "MarkDuplicates"
            ],
            "arguments": [
                {
                    "prefix": "--java-options",
                    "position": -1,
                    "valueFrom": "$(\"-Xmx{memory}G {compression} {otherargs}\".replace(/\\{memory\\}/g, (([inputs.runtime_memory, 8, 4].filter(function (inner) { return inner != null })[0] * 3) / 4)).replace(/\\{compression\\}/g, (inputs.compression_level != null) ? (\"-Dsamjdk.compress_level=\" + inputs.compression_level) : \"\").replace(/\\{otherargs\\}/g, [inputs.javaOptions, []].filter(function (inner) { return inner != null })[0].join(\" \")))"
                }
            ],
            "id": "#Gatk4MarkDuplicates_4_1_3_0.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "GATK4: Merge SAM Files",
            "doc": "Merges multiple SAM/BAM files into one file",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.1.3.0"
                }
            ],
            "inputs": [
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/javaOptions",
                    "label": "javaOptions",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/compression_level",
                    "label": "compression_level",
                    "doc": "Compression level for all compressed files created (e.g. BAM and VCF). Default value: 2.",
                    "type": [
                        "int",
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/bams",
                    "label": "bams",
                    "doc": "The SAM/BAM file to sort.",
                    "type": {
                        "type": "array",
                        "inputBinding": {
                            "prefix": "-I"
                        },
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/sampleName",
                    "label": "sampleName",
                    "doc": "Used for naming purposes only",
                    "type": [
                        "string",
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/outputFilename",
                    "label": "outputFilename",
                    "doc": "SAM/BAM file to write merged result to",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.merged.bam",
                    "inputBinding": {
                        "prefix": "-O",
                        "position": 10,
                        "valueFrom": "$(inputs.sampleName ? inputs.sampleName : \"generated\").merged.bam"
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/argumentsFile",
                    "label": "argumentsFile",
                    "doc": "read one or more arguments files and add them to the command line",
                    "type": [
                        {
                            "type": "array",
                            "items": "File"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--arguments_file",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/assumeSorted",
                    "label": "assumeSorted",
                    "doc": "If true, assume that the input files are in the same sort order as the requested output sort order, even if their headers say otherwise.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-AS"
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/comment",
                    "label": "comment",
                    "doc": "Comment(s) to include in the merged output file's header.",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-CO"
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/mergeSequenceDictionaries",
                    "label": "mergeSequenceDictionaries",
                    "doc": "Merge the sequence dictionaries",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-MSD"
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/sortOrder",
                    "label": "sortOrder",
                    "doc": "The --SORT_ORDER argument is an enumerated type (SortOrder), which can have one of the following values: [unsorted, queryname, coordinate, duplicate, unknown]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-SO",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/useThreading",
                    "label": "useThreading",
                    "doc": "Option to create a background thread to encode, compress and write to disk the output file. The threaded version uses about 20% more CPU and decreases runtime by ~20% when writing out a compressed BAM file.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--USE_THREADING"
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/compressionLevel",
                    "label": "compressionLevel",
                    "doc": "Compression level for all compressed files created (e.g. BAM and GELI).",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--COMPRESSION_LEVEL",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/createIndex",
                    "label": "createIndex",
                    "doc": "Whether to create a BAM index when writing a coordinate-sorted BAM file.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--CREATE_INDEX",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/createMd5File",
                    "label": "createMd5File",
                    "doc": "Whether to create an MD5 digest for any BAM or FASTQ files created.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--CREATE_MD5_FILE",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/maxRecordsInRam",
                    "label": "maxRecordsInRam",
                    "doc": "When writing SAM files that need to be sorted, this will specify the number of records stored in RAM before spilling to disk. Increasing this number reduces the number of file handles needed to sort a SAM file, and increases the amount of RAM needed.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--MAX_RECORDS_IN_RAM",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/quiet",
                    "label": "quiet",
                    "doc": "Whether to suppress job-summary info on System.err.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--QUIET",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/reference",
                    "label": "reference",
                    "doc": "Reference sequence file.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "secondaryFiles": [
                        ".fai",
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        "^.dict"
                    ],
                    "inputBinding": {
                        "prefix": "--reference",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/tmpDir",
                    "label": "tmpDir",
                    "doc": "Undocumented option",
                    "type": "string",
                    "default": "/tmp/",
                    "inputBinding": {
                        "prefix": "--TMP_DIR",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/useJdkDeflater",
                    "label": "useJdkDeflater",
                    "doc": "Whether to use the JdkDeflater (as opposed to IntelDeflater)",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--use_jdk_deflater",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/useJdkInflater",
                    "label": "useJdkInflater",
                    "doc": "Whether to use the JdkInflater (as opposed to IntelInflater)",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--use_jdk_inflater",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/validationStringency",
                    "label": "validationStringency",
                    "doc": "Validation stringency for all SAM files read by this program. Setting stringency to SILENT can improve performance when processing a BAM file in which variable-length data (read, qualities, tags) do not otherwise need to be decoded.The --VALIDATION_STRINGENCY argument is an enumerated type (ValidationStringency), which can have one of the following values: [STRICT, LENIENT, SILENT]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--VALIDATION_STRINGENCY",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/verbosity",
                    "label": "verbosity",
                    "doc": "The --verbosity argument is an enumerated type (LogLevel), which can have one of the following values: [ERROR, WARNING, INFO, DEBUG]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--verbosity",
                        "position": 11
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl/Gatk4MergeSamFiles/out",
                    "label": "out",
                    "type": "File",
                    "secondaryFiles": [
                        "${\n\n        function resolveSecondary(base, secPattern) {\n          if (secPattern[0] == \"^\") {\n            var spl = base.split(\".\");\n            var endIndex = spl.length > 1 ? spl.length - 1 : 1;\n            return resolveSecondary(spl.slice(undefined, endIndex).join(\".\"), secPattern.slice(1));\n          }\n          return base + secPattern\n        }\n        return [\n                {\n                    path: resolveSecondary(self.path, \"^.bai\"),\n                    basename: resolveSecondary(self.basename, \".bai\"),\n                    class: \"File\",\n                }\n        ];\n\n}"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.sampleName ? inputs.sampleName : \"generated\").merged.bam",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": [
                "gatk",
                "MergeSamFiles"
            ],
            "arguments": [
                {
                    "prefix": "--java-options",
                    "position": -1,
                    "valueFrom": "$(\"-Xmx{memory}G {compression} {otherargs}\".replace(/\\{memory\\}/g, (([inputs.runtime_memory, 8, 4].filter(function (inner) { return inner != null })[0] * 3) / 4)).replace(/\\{compression\\}/g, (inputs.compression_level != null) ? (\"-Dsamjdk.compress_level=\" + inputs.compression_level) : \"\").replace(/\\{otherargs\\}/g, [inputs.javaOptions, []].filter(function (inner) { return inner != null })[0].join(\" \")))"
                }
            ],
            "id": "#Gatk4MergeSamFiles_4_1_3_0.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "GATK4: SortSAM",
            "doc": "Sorts a SAM/BAM/CRAM file.",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.1.2.0"
                }
            ],
            "inputs": [
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/javaOptions",
                    "label": "javaOptions",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/compression_level",
                    "label": "compression_level",
                    "doc": "Compression level for all compressed files created (e.g. BAM and VCF). Default value: 2.",
                    "type": [
                        "int",
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/bam",
                    "label": "bam",
                    "doc": "The SAM/BAM/CRAM file to sort.",
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-I",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/outputFilename",
                    "label": "outputFilename",
                    "doc": "The sorted SAM/BAM/CRAM output file.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.sorted.bam",
                    "inputBinding": {
                        "prefix": "-O",
                        "position": 10,
                        "valueFrom": "$(inputs.bam.basename).sorted.bam"
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/sortOrder",
                    "label": "sortOrder",
                    "doc": "The --SORT_ORDER argument is an enumerated type (SortOrder), which can have one of the following values: [unsorted, queryname, coordinate, duplicate, unknown]",
                    "type": "string",
                    "inputBinding": {
                        "prefix": "-SO",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/argumentsFile",
                    "label": "argumentsFile",
                    "doc": "read one or more arguments files and add them to the command line",
                    "type": [
                        {
                            "type": "array",
                            "inputBinding": {
                                "prefix": "--arguments_file"
                            },
                            "items": "File"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/compressionLevel",
                    "label": "compressionLevel",
                    "doc": "Compression level for all compressed files created (e.g. BAM and GELI).",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--COMPRESSION_LEVEL",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/createIndex",
                    "label": "createIndex",
                    "doc": "Whether to create a BAM index when writing a coordinate-sorted BAM file.",
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "prefix": "--CREATE_INDEX",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/createMd5File",
                    "label": "createMd5File",
                    "doc": "Whether to create an MD5 digest for any BAM or FASTQ files created.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--CREATE_MD5_FILE",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/maxRecordsInRam",
                    "label": "maxRecordsInRam",
                    "doc": "When writing SAM files that need to be sorted, this will specify the number of records stored in RAM before spilling to disk. Increasing this number reduces the number of file handles needed to sort a SAM file, and increases the amount of RAM needed.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--MAX_RECORDS_IN_RAM",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/quiet",
                    "label": "quiet",
                    "doc": "Whether to suppress job-summary info on System.err.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--QUIET",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/reference",
                    "label": "reference",
                    "doc": "Reference sequence file.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "secondaryFiles": [
                        ".fai",
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        "^.dict"
                    ],
                    "inputBinding": {
                        "prefix": "--reference",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/tmpDir",
                    "label": "tmpDir",
                    "doc": "Undocumented option",
                    "type": "string",
                    "default": "/tmp/",
                    "inputBinding": {
                        "prefix": "--TMP_DIR",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/useJdkDeflater",
                    "label": "useJdkDeflater",
                    "doc": "Whether to use the JdkDeflater (as opposed to IntelDeflater)",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--use_jdk_deflater",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/useJdkInflater",
                    "label": "useJdkInflater",
                    "doc": "Whether to use the JdkInflater (as opposed to IntelInflater)",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--use_jdk_inflater",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/validationStringency",
                    "label": "validationStringency",
                    "doc": "Validation stringency for all SAM files read by this program. Setting stringency to SILENT can improve performance when processing a BAM file in which variable-length data (read, qualities, tags) do not otherwise need to be decoded.The --VALIDATION_STRINGENCY argument is an enumerated type (ValidationStringency), which can have one of the following values: [STRICT, LENIENT, SILENT]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--VALIDATION_STRINGENCY",
                        "position": 11
                    }
                },
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/verbosity",
                    "label": "verbosity",
                    "doc": "The --verbosity argument is an enumerated type (LogLevel), which can have one of the following values: [ERROR, WARNING, INFO, DEBUG]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--verbosity",
                        "position": 11
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#Gatk4SortSam_4_1_2_0.cwl/Gatk4SortSam/out",
                    "label": "out",
                    "type": "File",
                    "secondaryFiles": [
                        "${\n\n        function resolveSecondary(base, secPattern) {\n          if (secPattern[0] == \"^\") {\n            var spl = base.split(\".\");\n            var endIndex = spl.length > 1 ? spl.length - 1 : 1;\n            return resolveSecondary(spl.slice(undefined, endIndex).join(\".\"), secPattern.slice(1));\n          }\n          return base + secPattern\n        }\n        return [\n                {\n                    path: resolveSecondary(self.path, \"^.bai\"),\n                    basename: resolveSecondary(self.basename, \".bai\"),\n                    class: \"File\",\n                }\n        ];\n\n}"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.bam.basename).sorted.bam",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": [
                "gatk",
                "SortSam"
            ],
            "arguments": [
                {
                    "prefix": "--java-options",
                    "position": -1,
                    "valueFrom": "$(\"-Xmx{memory}G {compression} {otherargs}\".replace(/\\{memory\\}/g, (([inputs.runtime_memory, 8, 4].filter(function (inner) { return inner != null })[0] * 3) / 4)).replace(/\\{compression\\}/g, (inputs.compression_level != null) ? (\"-Dsamjdk.compress_level=\" + inputs.compression_level) : \"\").replace(/\\{otherargs\\}/g, [inputs.javaOptions, []].filter(function (inner) { return inner != null })[0].join(\" \")))"
                }
            ],
            "id": "#Gatk4SortSam_4_1_2_0.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "GenerateGenomeFileForBedtoolsCoverage",
            "doc": "",
            "requirements": [
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": [
                        {
                            "entryname": "GenerateGenomeFileForBedtoolsCoverage-script.py",
                            "entry": "\nimport argparse, json, sys\nfrom typing import Optional, List, Dict, Any\ncli = argparse.ArgumentParser(\"Argument parser for Janis PythonTool\")\ncli.add_argument(\"--reference\", type=str, required=True)\ncli.add_argument(\"--output_filename\", type=str, help='Filename to output to')\n\nFastaDict = str\nString = str\nFilename = str\nInt = int\nFloat = float\nDouble = float\nBoolean = str\nFile = str\nDirectory = str\nStdout = str\nStderr = str\nArray = List\nclass PythonTool:\n    File = str\n    Directory = str\n\n\n\ndef code_block(\n    reference: FastaDict, output_filename: str = \"genome_file.txt\"\n) -> Dict[str, Any]:\n    \"\"\"\n    :param reference: Reference file to generate genome for (must have ^.dict) pattern\n    :param output_filename: Filename to output to\n    \"\"\"\n    from re import sub\n\n    ref_dict = sub(\"\\.fa(sta)?$\", \".dict\", reference)\n\n    with open(ref_dict) as inp, open(output_filename, \"w+\") as out:\n        for line in inp:\n            if not line.startswith(\"@SQ\"):\n                continue\n            pieces = line.split(\"\\t\")\n            chrom = pieces[1].replace(\"SN:\", \"\")\n            length = pieces[2].replace(\"LN:\", \"\")\n\n            out.write(f\"{chrom}\\t{length}\\n\")\n\n        return {\"out\": output_filename}\n\n\ntry:\n    args = cli.parse_args()\n    result = code_block(reference=args.reference, output_filename=args.output_filename)\n\n    from os import getcwd, path\n    cwd = getcwd()\n    def prepare_file_or_directory_type(file_or_directory, value):\n        if value is None:\n            return None\n        if isinstance(value, list):\n            return [prepare_file_or_directory_type(file_or_directory, v) for v in value]\n        return {\n            \"class\": file_or_directory,\n            \"path\": path.join(cwd, value)\n        }\n    result[\"out\"] = prepare_file_or_directory_type(\"File\", result[\"out\"])\n    print(json.dumps(result))\nexcept Exception as e:\n    print(str(e), file=sys.stderr)\n    raise\n"
                        }
                    ]
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "python:3.8.1"
                }
            ],
            "inputs": [
                {
                    "id": "#GenerateGenomeFileForBedtoolsCoverage_v0_1_0.cwl/GenerateGenomeFileForBedtoolsCoverage/reference",
                    "label": "reference",
                    "type": "File",
                    "secondaryFiles": [
                        "^.dict"
                    ],
                    "inputBinding": {
                        "prefix": "--reference"
                    }
                },
                {
                    "id": "#GenerateGenomeFileForBedtoolsCoverage_v0_1_0.cwl/GenerateGenomeFileForBedtoolsCoverage/output_filename",
                    "label": "output_filename",
                    "doc": "Filename to output to",
                    "type": "string",
                    "default": "genome_file.txt",
                    "inputBinding": {
                        "prefix": "--output_filename"
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#GenerateGenomeFileForBedtoolsCoverage_v0_1_0.cwl/GenerateGenomeFileForBedtoolsCoverage/out",
                    "label": "out",
                    "doc": "Genome file for BedToolsCoverage",
                    "type": "File"
                }
            ],
            "stdout": "cwl.output.json",
            "stderr": "python-capture.stderr",
            "baseCommand": [
                "python",
                "GenerateGenomeFileForBedtoolsCoverage-script.py"
            ],
            "id": "#GenerateGenomeFileForBedtoolsCoverage_v0_1_0.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "ParseFastqcAdaptors",
            "doc": "",
            "requirements": [
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": [
                        {
                            "entryname": "ParseFastqcAdaptors-script.py",
                            "entry": "\nimport argparse, json, sys\nfrom typing import Optional, List, Dict, Any\ncli = argparse.ArgumentParser(\"Argument parser for Janis PythonTool\")\ncli.add_argument(\"--fastqc_datafiles\", nargs='+', default=[], type=str)\ncli.add_argument(\"--cutadapt_adaptors_lookup\", type=str, help='Specifies a file which contains the list of adapter sequences which will\\nbe explicity searched against the library. The file must contain sets of named adapters in\\nthe form name[tab]sequence. Lines prefixed with a hash will be ignored.')\n\nArray = List\nFile = str\nString = str\nFilename = str\nInt = int\nFloat = float\nDouble = float\nBoolean = str\nDirectory = str\nStdout = str\nStderr = str\nclass PythonTool:\n    File = str\n    Directory = str\n\n\n\ndef code_block(\n    fastqc_datafiles: List[File], cutadapt_adaptors_lookup: Optional[File]\n):\n    \"\"\"\n\n    :param fastqc_datafiles:\n\n    :param cutadapt_adaptors_lookup: Specifies a file which contains the list of adapter sequences which will\n        be explicity searched against the library. The file must contain sets of named adapters in\n        the form name[tab]sequence. Lines prefixed with a hash will be ignored.\n    :return:\n    \"\"\"\n    if not cutadapt_adaptors_lookup:\n        return {\"adaptor_sequences\": []}\n\n    import mmap, re, csv\n    from io import StringIO\n    from sys import stderr\n\n    def get_overrepresented_text(f):\n        \"\"\"\n        Get the table \"Overrepresented sequences\" within the fastqc_data.txt\n        \"\"\"\n        adapt_section_query = (\n            br\"(?s)>>Overrepresented sequences\\t\\S+\\n(.*?)>>END_MODULE\"\n        )\n        # fastqc_datafile could be fairly large, so we'll use mmap, and then\n        with open(f) as f, mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ) as fp:\n            overrepresented_sequences_match = re.search(adapt_section_query, fp)\n            if overrepresented_sequences_match is None:\n                raise Exception(\n                    f\"Couldn't find query ('{adapt_section_query.decode('utf8')}') in {fastqc_datafiles}\"\n                )\n\n            return overrepresented_sequences_match.groups()[0].decode(\"utf8\")\n\n    def parse_tsv_table(tbl: str, skip_headers):\n        \"\"\"\n        Parse a TSV table from a string using csvreader\n        \"\"\"\n\n        rd = csv.reader(StringIO(tbl), delimiter=\"\\t\", quotechar='\"')\n        ret = list(rd)\n        if len(ret) == 0:\n            return ret\n        if skip_headers:\n            ret.pop(0)  # discard headers\n        return ret\n\n    def get_cutadapt_map():\n        \"\"\"\n        Helper method to parse the file 'cutadapt_adaptors_lookup' with\n        format: 'name[tab]sequence' into the dictionary: '{ sequence: name }'\n        \"\"\"\n        cutadapt_map = {}\n        with open(cutadapt_adaptors_lookup) as fp:\n            for row in fp:\n                st = row.strip()\n                if not st or st.startswith(\"#\"):\n                    continue\n\n                # In reality, the format is $name[\\t+]$seqence (more than one tab)\n                # so we'll just split on a tab, and remove all the empty elements.\n                split = [f for f in st.split(\"\\t\") if bool(f) and len(f) > 0]\n\n                # Invalid format for line, so skip it.\n                if len(split) != 2:\n                    print(\n                        f\"Skipping cutadapt line '{st}' as irregular elements ({len(split)})\",\n                        file=stderr,\n                    )\n                    continue\n\n                # reverse the order from name[tab]sequence to { sequence: tab }\n                cutadapt_map[split[1]] = split[0]\n        return cutadapt_map\n\n    # Start doing the work\n    adaptor_ids = set()\n    for fastqcfile in fastqc_datafiles:\n        text = get_overrepresented_text(fastqcfile)\n        adaptor_ids = adaptor_ids.union(\n            set(a[0] for a in parse_tsv_table(text, skip_headers=True))\n        )\n\n    adaptor_sequences = []\n\n    if adaptor_ids:\n        cutadapt_map = get_cutadapt_map()\n        for aid in adaptor_ids:\n            if aid in cutadapt_map:\n                print(\n                    f\"Identified sequence '{aid}' as '{cutadapt_map.get(aid)}' in lookup\",\n                    file=stderr,\n                )\n                adaptor_sequences.append(aid)\n\n            else:\n                print(\n                    f\"Couldn't find a corresponding sequence for '{aid}' in lookup map\",\n                    file=stderr,\n                )\n\n    return {\"adaptor_sequences\": adaptor_sequences}\n\n\ntry:\n    args = cli.parse_args()\n    result = code_block(fastqc_datafiles=args.fastqc_datafiles, cutadapt_adaptors_lookup=args.cutadapt_adaptors_lookup)\n\n    from os import getcwd, path\n    cwd = getcwd()\n    def prepare_file_or_directory_type(file_or_directory, value):\n        if value is None:\n            return None\n        if isinstance(value, list):\n            return [prepare_file_or_directory_type(file_or_directory, v) for v in value]\n        return {\n            \"class\": file_or_directory,\n            \"path\": path.join(cwd, value)\n        }\n    print(json.dumps(result))\nexcept Exception as e:\n    print(str(e), file=sys.stderr)\n    raise\n"
                        }
                    ]
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "python:3.8.1"
                }
            ],
            "inputs": [
                {
                    "id": "#ParseFastqcAdaptors_v0_1_0.cwl/ParseFastqcAdaptors/fastqc_datafiles",
                    "label": "fastqc_datafiles",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "prefix": "--fastqc_datafiles"
                    }
                },
                {
                    "id": "#ParseFastqcAdaptors_v0_1_0.cwl/ParseFastqcAdaptors/cutadapt_adaptors_lookup",
                    "label": "cutadapt_adaptors_lookup",
                    "doc": "Specifies a file which contains the list of adapter sequences which will\nbe explicity searched against the library. The file must contain sets of named adapters in\nthe form name[tab]sequence. Lines prefixed with a hash will be ignored.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--cutadapt_adaptors_lookup"
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#ParseFastqcAdaptors_v0_1_0.cwl/ParseFastqcAdaptors/adaptor_sequences",
                    "label": "adaptor_sequences",
                    "type": {
                        "type": "array",
                        "items": "string"
                    }
                }
            ],
            "stdout": "cwl.output.json",
            "stderr": "python-capture.stderr",
            "baseCommand": [
                "python",
                "ParseFastqcAdaptors-script.py"
            ],
            "id": "#ParseFastqcAdaptors_v0_1_0.cwl"
        },
        {
            "class": "Workflow",
            "label": "Performance summary workflow (whole genome)",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                }
            ],
            "inputs": [
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bam",
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ]
                },
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/sample_name",
                    "type": "string"
                },
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/genome_file",
                    "type": "File"
                },
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/samtoolsview_doNotOutputAlignmentsWithBitsSet",
                    "doc": "Do not output alignments with any bits set in INT present in the FLAG field. INT can be specified in hex by beginning with `0x' (i.e. /^0x[0-9A-F]+/) or in octal by beginning with `0' (i.e. /^0[0-7]+/) [0].",
                    "type": "string",
                    "default": "0x400"
                },
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary_genome",
                    "doc": "calculate statistics for whole genome data.--target_flagstat must not be speicified",
                    "type": "boolean",
                    "default": true
                }
            ],
            "outputs": [
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performanceSummaryOut",
                    "type": "File",
                    "outputSource": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary/out"
                }
            ],
            "steps": [
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/gatk4collectinsertsizemetrics",
                    "label": "GATK4: CollectInsertSizeMetrics",
                    "in": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/gatk4collectinsertsizemetrics/bam",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bam"
                        }
                    ],
                    "run": "#Gatk4CollectInsertSizeMetrics_4_1_3_0.cwl",
                    "out": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/gatk4collectinsertsizemetrics/out"
                        },
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/gatk4collectinsertsizemetrics/outHistogram"
                        }
                    ]
                },
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bamflagstat",
                    "label": "SamTools: Flagstat",
                    "in": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bamflagstat/bam",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bam"
                        }
                    ],
                    "run": "#SamToolsFlagstat_1_9_0.cwl",
                    "out": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bamflagstat/out"
                        }
                    ]
                },
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/samtoolsview",
                    "label": "SamTools: View",
                    "in": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/samtoolsview/doNotOutputAlignmentsWithBitsSet",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/samtoolsview_doNotOutputAlignmentsWithBitsSet"
                        },
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/samtoolsview/sam",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bam"
                        }
                    ],
                    "run": "#SamToolsView_1_9_0.cwl",
                    "out": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/samtoolsview/out"
                        }
                    ]
                },
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/rmdupbamflagstat",
                    "label": "SamTools: Flagstat",
                    "in": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/rmdupbamflagstat/bam",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/samtoolsview/out"
                        }
                    ],
                    "run": "#SamToolsFlagstat_1_9_0.cwl",
                    "out": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/rmdupbamflagstat/out"
                        }
                    ]
                },
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bedtoolsgenomecoveragebed",
                    "label": "BEDTools: genomeCoverageBed",
                    "in": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bedtoolsgenomecoveragebed/inputBam",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/samtoolsview/out"
                        },
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bedtoolsgenomecoveragebed/genome",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/genome_file"
                        }
                    ],
                    "run": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl",
                    "out": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bedtoolsgenomecoveragebed/out"
                        }
                    ]
                },
                {
                    "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary",
                    "label": "Performance Summary",
                    "in": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary/flagstat",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bamflagstat/out"
                        },
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary/collectInsertSizeMetrics",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/gatk4collectinsertsizemetrics/out"
                        },
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary/coverage",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/bedtoolsgenomecoveragebed/out"
                        },
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary/outputPrefix",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/sample_name"
                        },
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary/rmdupFlagstat",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/rmdupbamflagstat/out"
                        },
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary/genome",
                            "source": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary_genome"
                        }
                    ],
                    "run": "#performanceSummary_0_0_7.cwl",
                    "out": [
                        {
                            "id": "#PerformanceSummaryGenome_v0_1_0.cwl/PerformanceSummaryGenome/performancesummary/out"
                        }
                    ]
                }
            ],
            "id": "#PerformanceSummaryGenome_v0_1_0.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "SamTools: Flagstat",
            "doc": "Does a full pass through the input file to calculate and print statistics to stdout.\n\nProvides counts for each of 13 categories based primarily on bit flags in the FLAG field. Each category in the output is broken down into QC pass and QC fail. In the default output format, these are presented as \"#PASS + #FAIL\" followed by a description of the category.\n\nThe first row of output gives the total number of reads that are QC pass and fail (according to flag bit 0x200). For example:\n\n122 + 28 in total (QC-passed reads + QC-failed reads)\n\nWhich would indicate that there are a total of 150 reads in the input file, 122 of which are marked as QC pass and 28 of which are marked as \"not passing quality controls\"\n\nFollowing this, additional categories are given for reads which are:\n\nsecondary     0x100 bit set\n\nsupplementary     0x800 bit set\n\nduplicates     0x400 bit set\n\nmapped     0x4 bit not set\n\npaired in sequencing     0x1 bit set\n\nread1     both 0x1 and 0x40 bits set\n\nread2     both 0x1 and 0x80 bits set\n\nproperly paired     both 0x1 and 0x2 bits set and 0x4 bit not set\n\nwith itself and mate mapped     0x1 bit set and neither 0x4 nor 0x8 bits set\n\nsingletons     both 0x1 and 0x8 bits set and bit 0x4 not set\n\nAnd finally, two rows are given that additionally filter on the reference name (RNAME), mate reference name (MRNM), and mapping quality (MAPQ) fields:\n\nwith mate mapped to a different chr     0x1 bit set and neither 0x4 nor 0x8 bits set and MRNM not equal to RNAME\n\nwith mate mapped to a different chr (mapQ>=5)     0x1 bit set and neither 0x4 nor 0x8 bits set and MRNM not equal to RNAME and MAPQ >= 5)",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/samtools:1.9--h8571acd_11"
                }
            ],
            "inputs": [
                {
                    "id": "#SamToolsFlagstat_1_9_0.cwl/SamToolsFlagstat/bam",
                    "label": "bam",
                    "type": "File",
                    "inputBinding": {
                        "position": 10
                    }
                },
                {
                    "id": "#SamToolsFlagstat_1_9_0.cwl/SamToolsFlagstat/threads",
                    "label": "threads",
                    "doc": "Number of BAM compression threads to use in addition to main thread [0].",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-@",
                        "position": 5
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#SamToolsFlagstat_1_9_0.cwl/SamToolsFlagstat/out",
                    "label": "out",
                    "type": "File",
                    "outputBinding": {
                        "glob": "_stdout"
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": [
                "samtools",
                "flagstat"
            ],
            "arguments": [],
            "id": "#SamToolsFlagstat_1_9_0.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "SamTools: View",
            "doc": "Ensure SAMTOOLS.SORT is inheriting from parent metadata\n        \n---------------------------------------------------------------------------------------------------\n    \nWith no options or regions specified, prints all alignments in the specified input alignment file \n(in SAM, BAM, or CRAM format) to standard output in SAM format (with no header).\n\nYou may specify one or more space-separated region specifications after the input filename to \nrestrict output to only those alignments which overlap the specified region(s). \nUse of region specifications requires a coordinate-sorted and indexed input file (in BAM or CRAM format).",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/samtools:1.9--h8571acd_11"
                }
            ],
            "inputs": [
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/cramOutput",
                    "label": "cramOutput",
                    "doc": "Output in the CRAM format (requires -T).",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-C",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/compressedBam",
                    "label": "compressedBam",
                    "doc": "Enable fast BAM compression (implies -b).",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-1",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/uncompressedBam",
                    "label": "uncompressedBam",
                    "doc": "Output uncompressed BAM. This option saves time spent on compression/decompression and is thus preferred when the output is piped to another samtools command.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-u",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/onlyOutputHeader",
                    "label": "onlyOutputHeader",
                    "doc": "Output the header only.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-H",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/countAlignments",
                    "label": "countAlignments",
                    "doc": "Instead of printing the alignments, only count them and print the total number. All filter options, such as -f, -F, and -q, are taken into account.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-c",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/writeAlignments",
                    "label": "writeAlignments",
                    "doc": "Write alignments that are not selected by the various filter options to FILE. When this option is used, all alignments (or all alignments intersecting the regions specified) are written to either the output file or this file, but never both.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-U",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/inputTSV",
                    "label": "inputTSV",
                    "doc": "A tab-delimited FILE. Each line must contain the reference name in the first column and the length of the reference in the second column, with one line for each distinct reference. Any additional fields beyond the second column are ignored. This file also defines the order of the reference sequences in sorting. If you run: `samtools faidx <ref.fa>', the resulting index file <ref.fa>.fai can be used as this FILE.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-t",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/onlyOverlapping",
                    "label": "onlyOverlapping",
                    "doc": "Only output alignments overlapping the input BED FILE [null].",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-L",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/useMultiRegionIterator",
                    "label": "useMultiRegionIterator",
                    "doc": "Use the multi-region iterator on the union of the BED file and command-line region arguments. This avoids re-reading the same regions of files so can sometimes be much faster. Note this also removes duplicate sequences. Without this a sequence that overlaps multiple regions specified on the command line will be reported multiple times.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-M",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/outputAlignmentsInReadGroup",
                    "label": "outputAlignmentsInReadGroup",
                    "doc": "Output alignments in read group STR [null]. Note that records with no RG tag will also be output when using this option. This behaviour may change in a future release.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-r",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/outputAlignmentsInFileReadGroups",
                    "label": "outputAlignmentsInFileReadGroups",
                    "doc": "Output alignments in read groups listed in FILE [null]. Note that records with no RG tag will also be output when using this option. This behaviour may change in a future release.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-R",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/mapqThreshold",
                    "label": "mapqThreshold",
                    "doc": "Skip alignments with MAPQ smaller than INT [0].",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-q",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/outputAlignmentsInLibrary",
                    "label": "outputAlignmentsInLibrary",
                    "doc": "Only output alignments in library STR [null].",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-l",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/outputAlignmentsMeetingCIGARThreshold",
                    "label": "outputAlignmentsMeetingCIGARThreshold",
                    "doc": "Only output alignments with number of CIGAR bases consuming query sequence \u2265 INT [0]",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-m",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/outputAlignmentsWithBitsSet",
                    "label": "outputAlignmentsWithBitsSet",
                    "doc": "Only output alignments with all bits set in INT present in the FLAG field. INT can be specified in hex by beginning with `0x' (i.e. /^0x[0-9A-F]+/) or in octal by beginning with `0' (i.e. /^0[0-7]+/) [0].",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-f",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/doNotOutputAlignmentsWithBitsSet",
                    "label": "doNotOutputAlignmentsWithBitsSet",
                    "doc": "Do not output alignments with any bits set in INT present in the FLAG field. INT can be specified in hex by beginning with `0x' (i.e. /^0x[0-9A-F]+/) or in octal by beginning with `0' (i.e. /^0[0-7]+/) [0].",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-F",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/doNotOutputAlignmentsWithAllBitsSet",
                    "label": "doNotOutputAlignmentsWithAllBitsSet",
                    "doc": "Do not output alignments with all bits set in INT present in the FLAG field. This is the opposite of -f such that -f12 -G12 is the same as no filtering at all. INT can be specified in hex by beginning with `0x' (i.e. /^0x[0-9A-F]+/) or in octal by beginning with `0' (i.e. /^0[0-7]+/) [0].",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-G",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/readTagToExclude",
                    "label": "readTagToExclude",
                    "doc": "Read tag to exclude from output (repeatable) [null]",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-x",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/collapseBackwardCIGAR",
                    "label": "collapseBackwardCIGAR",
                    "doc": "Collapse the backward CIGAR operation.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-B",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/subsamplingProportion",
                    "label": "subsamplingProportion",
                    "doc": "Output only a proportion of the input alignments. This subsampling acts in the same way on all of the alignment records in the same template or read pair, so it never keeps a read but not its mate. The integer and fractional parts of the -s INT.FRAC option are used separately: the part after the decimal point sets the fraction of templates/pairs to be kept, while the integer part is used as a seed that influences which subset of reads is kept.",
                    "type": [
                        "float",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-s",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/threads",
                    "label": "threads",
                    "doc": "Number of BAM compression threads to use in addition to main thread [0].",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-@",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/sam",
                    "label": "sam",
                    "type": "File",
                    "inputBinding": {
                        "position": 10
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/reference",
                    "label": "reference",
                    "doc": "A FASTA format reference FILE, optionally compressed by bgzip and ideally indexed by samtools faidx. If an index is not present, one will be generated for you.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "secondaryFiles": [
                        ".fai",
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        "^.dict"
                    ],
                    "inputBinding": {
                        "prefix": "-T",
                        "position": 6
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/outputFilename",
                    "label": "outputFilename",
                    "doc": "Output to FILE [stdout].",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.bam",
                    "inputBinding": {
                        "prefix": "-o",
                        "position": 5
                    }
                },
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/regions",
                    "label": "regions",
                    "doc": "Region specifications after the input filename to restrict output to only those alignments which overlap the specified region(s). Use of region specifications requires a coordinate-sorted and indexed input file (in BAM or CRAM format)",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ],
                    "inputBinding": {
                        "position": 11
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#SamToolsView_1_9_0.cwl/SamToolsView/out",
                    "label": "out",
                    "type": "File",
                    "outputBinding": {
                        "glob": "generated.bam",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": [
                "samtools",
                "view"
            ],
            "arguments": [
                {
                    "position": 2,
                    "valueFrom": "-S"
                },
                {
                    "position": 3,
                    "valueFrom": "-h"
                },
                {
                    "position": 4,
                    "valueFrom": "-b"
                }
            ],
            "id": "#SamToolsView_1_9_0.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "BEDTools: genomeCoverageBed",
            "doc": "bedtools genomecov computes histograms (default), per-base reports (-d) and BEDGRAPH (-bg) summaries of feature coverage (e.g., aligned sequences) for a given genome. Note: 1. If using BED/GFF/VCF, the input (-i) file must be grouped by chromosome. A simple sort -k 1,1 in.bed > in.sorted.bed will suffice. Also, if using BED/GFF/VCF, one must provide a genome file via the -g argument. 2. If the input is in BAM (-ibam) format, the BAM file must be sorted by position. Using samtools sort aln.bam aln.sorted will suffice.",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/bedtools:2.29.2--hc088bd4_0"
                }
            ],
            "inputs": [
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/depth",
                    "label": "depth",
                    "doc": "Report the depth at each genome position (with one-based coordinates). Default behavior is to report a histogram.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-d"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/depthZero",
                    "label": "depthZero",
                    "doc": "Report the depth at each genome position (with zero-based coordinates). Reports only non-zero positions. Default behavior is to report a histogram.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-dz"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/BedGraphFormat",
                    "label": "BedGraphFormat",
                    "doc": "Report depth in BedGraph format. For details, see: genome.ucsc.edu/goldenPath/help/bedgraph.html",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-bg"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/BedGraphFormata",
                    "label": "BedGraphFormata",
                    "doc": "Report depth in BedGraph format, as above (-bg). However with this option, regions with zero coverage are also reported. This allows one to quickly extract all regions of a genome with 0  coverage by applying: 'grep -w 0$' to the output.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-bga"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/split",
                    "label": "split",
                    "doc": "Treat 'split' BAM or BED12 entries as distinct BED intervals when computing coverage. For BAM files, this uses the CIGAR 'N' and 'D' operations to infer the blocks for computing coverage. For BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds fields (i.e., columns 10,11,12).",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-split"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/strand",
                    "label": "strand",
                    "doc": "(STRING): can be + or -. Calculate coverage of intervals from a specific strand. With BED files, requires at least 6 columns (strand is column 6).",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-strand"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/pairEnd",
                    "label": "pairEnd",
                    "doc": "Calculate coverage of pair-end fragments. Works for BAM files only",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-pc"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/fragmentSize",
                    "label": "fragmentSize",
                    "doc": "Force to use provided fragment size instead of read length. Works for BAM files only",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-fs"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/du",
                    "label": "du",
                    "doc": "Change strand af the mate read (so both reads from the same strand) useful for strand specific. Works for BAM files only",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-du"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/fivePos",
                    "label": "fivePos",
                    "doc": "Calculate coverage of 5' positions (instead of entire interval).",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-5"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/threePos",
                    "label": "threePos",
                    "doc": "Calculate coverage of 3' positions (instead of entire interval).",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-3"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/max",
                    "label": "max",
                    "doc": "Combine all positions with a depth >= max into a single bin in the histogram. Irrelevant for -d and -bedGraph",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-max"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/scale",
                    "label": "scale",
                    "doc": "Scale the coverage by a constant factor. Each coverage value is multiplied by this factor before being reported. Useful for normalizing coverage by, e.g., reads per million (RPM). Default is 1.0; i.e., unscaled.",
                    "type": [
                        "float",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-scale"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/trackline",
                    "label": "trackline",
                    "doc": "Adds a UCSC/Genome-Browser track line definition in the first line of the output. - See here for more details about track line definition: http://genome.ucsc.edu/goldenPath/help/bedgraph.html - NOTE: When adding a trackline definition, the output BedGraph can be easily uploaded to the Genome Browser as a custom track, BUT CAN NOT be converted into a BigWig file (w/o removing the first line).",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-trackline"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/trackopts",
                    "label": "trackopts",
                    "doc": "Writes additional track line definition parameters in the first line. - Example: -trackopts 'name=\"My Track\" visibility=2 color=255,30,30' Note the use of single-quotes if you have spaces in your parameters.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-trackopts"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/inputBam",
                    "label": "inputBam",
                    "doc": "Input bam file. Note: BAM _must_ be sorted by position. A 'samtools sort <BAM>' should suffice.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-ibam"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/inputBed",
                    "label": "inputBed",
                    "doc": "Input bed file. Must be grouped by chromosome. A simple 'sort -k 1,1 <BED> > <BED>.sorted' will suffice.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-iBed"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/inputFile",
                    "label": "inputFile",
                    "doc": "Input file, can be gff/vcf.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-i"
                    }
                },
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/genome",
                    "label": "genome",
                    "doc": "Genome file. The genome file should tab delimited and structured as follows: <chromName><TAB><chromSize>.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-g"
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl/bedtoolsgenomeCoverageBed/out",
                    "label": "out",
                    "type": "File",
                    "outputBinding": {
                        "glob": "_stdout"
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": [
                "genomeCoverageBed"
            ],
            "arguments": [],
            "id": "#bedtoolsgenomeCoverageBed_v2_29_2.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "Cutadapt",
            "doc": "cutadapt version 2.4\nCopyright (C) 2010-2019 Marcel Martin <marcel.martin@scilifelab.se>\ncutadapt removes adapter sequences from high-throughput sequencing reads.\nUsage:\n    cutadapt -a ADAPTER [options] [-o output.fastq] input.fastq\nFor paired-end reads:\n    cutadapt -a ADAPT1 -A ADAPT2 [options] -o out1.fastq -p out2.fastq in1.fastq in2.fastq\nReplace \"ADAPTER\" with the actual sequence of your 3' adapter. IUPAC wildcard\ncharacters are supported. The reverse complement is *not* automatically\nsearched. All reads from input.fastq will be written to output.fastq with the\nadapter sequence removed. Adapter matching is error-tolerant. Multiple adapter\nsequences can be given (use further -a options), but only the best-matching\nadapter will be removed.\nInput may also be in FASTA format. Compressed input and output is supported and\nauto-detected from the file name (.gz, .xz, .bz2). Use the file name '-' for\nstandard input/output. Without the -o option, output is sent to standard output.\nCitation:\nMarcel Martin. Cutadapt removes adapter sequences from high-throughput\nsequencing reads. EMBnet.Journal, 17(1):10-12, May 2011.\nhttp://dx.doi.org/10.14806/ej.17.1.200\nRun \"cutadapt - -help\" to see all command-line options.\nSee https://cutadapt.readthedocs.io/ for full documentation.\n",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/cutadapt:2.1--py37h14c3975_0"
                }
            ],
            "inputs": [
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/fastq",
                    "label": "fastq",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 5
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/adapter",
                    "label": "adapter",
                    "doc": "Sequence of an adapter ligated to the 3' end (paired data: of the first read). The adapter and subsequent bases are trimmed. If a '$' character is appended ('anchoring'), the adapter is only found if it is a suffix of the read.",
                    "type": [
                        {
                            "type": "array",
                            "inputBinding": {
                                "prefix": "-a"
                            },
                            "items": "string"
                        },
                        "null"
                    ],
                    "inputBinding": {}
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/outputFilename",
                    "label": "outputFilename",
                    "doc": "Write trimmed reads to FILE. FASTQ or FASTA format is chosen depending on input. The summary report is sent to standard output. Use '{name}' in FILE to demultiplex reads into multiple files. Default: write to standard output",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated--R1.fastq.gz",
                    "inputBinding": {
                        "prefix": "-o"
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/secondReadFile",
                    "label": "secondReadFile",
                    "doc": "Write second read in a pair to FILE.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated--R2.fastq.gz",
                    "inputBinding": {
                        "prefix": "-p"
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/cores",
                    "label": "cores",
                    "doc": "(-j)  Number of CPU cores to use. Use 0 to auto-detect. Default: 1",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--cores",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/front",
                    "label": "front",
                    "doc": "(-g)  Sequence of an adapter ligated to the 5' end (paired data: of the first read). The adapter and any preceding bases are trimmed. Partial matches at the 5' end are allowed. If a '^' character is prepended ('anchoring'), the adapter is only found if it is a prefix of the read.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--front",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/anywhere",
                    "label": "anywhere",
                    "doc": "(-b)  Sequence of an adapter that may be ligated to the 5' or 3' end (paired data: of the first read). Both types of matches as described under -a und -g are allowed. If the first base of the read is part of the match, the behavior is as with -g, otherwise as with -a. This option is mostly for rescuing failed library preparations - do not use if you know which end your adapter was ligated to!",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--anywhere",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/errorRate",
                    "label": "errorRate",
                    "doc": "(-e)  Maximum allowed error rate as value between 0 and 1 (no. of errors divided by length of matching region). Default: 0.1 (=10%)",
                    "type": [
                        "float",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--error-rate",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/noIndels",
                    "label": "noIndels",
                    "doc": "Allow only mismatches in alignments. Default: allow both mismatches and indels",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--no-indels",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/times",
                    "label": "times",
                    "doc": "(-n)  Remove up to COUNT adapters from each read. Default: 1",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--times",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/overlap",
                    "label": "overlap",
                    "doc": "(-O)  Require MINLENGTH overlap between read and adapter for an adapter to be found. Default: 3",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--overlap",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/matchReadWildcards",
                    "label": "matchReadWildcards",
                    "doc": " Interpret IUPAC wildcards in reads. Default: False",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--match-read-wildcards",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/noMatchAdapterWildcards",
                    "label": "noMatchAdapterWildcards",
                    "doc": "(-N)  Do not interpret IUPAC wildcards in adapters.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--no-match-adapter-wildcards",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/action",
                    "label": "action",
                    "doc": "(trim,mask,lowercase,none}  What to do with found adapters. mask: replace with 'N' characters; lowercase: convert to lowercase; none: leave unchanged (useful with --discard-untrimmed). Default: trim",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--action",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/cut",
                    "label": "cut",
                    "doc": "(-u)  Remove bases from each read (first read only if paired). If LENGTH is positive, remove bases from the beginning. If LENGTH is negative, remove bases from the end. Can be used twice if LENGTHs have different signs. This is applied *before* adapter trimming.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--cut",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/nextseqTrim",
                    "label": "nextseqTrim",
                    "doc": " NextSeq-specific quality trimming (each read). Trims also dark cycles appearing as high-quality G bases.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--nextseq-trim",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/qualityCutoff",
                    "label": "qualityCutoff",
                    "doc": "(]3'CUTOFF, ]3'CUTOFF, -q)  Trim low-quality bases from 5' and/or 3' ends of each read before adapter removal. Applied to both reads if data is paired. If one value is given, only the 3' end is trimmed. If two comma-separated cutoffs are given, the 5' end is trimmed with the first cutoff, the 3' end with the second.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--quality-cutoff",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/qualityBase",
                    "label": "qualityBase",
                    "doc": "Assume that quality values in FASTQ are encoded as ascii(quality + N). This needs to be set to 64 for some old Illumina FASTQ files. Default: 33",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--quality-base",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/length",
                    "label": "length",
                    "doc": "(-l)  Shorten reads to LENGTH. Positive values remove bases at the end while negative ones remove bases at the beginning. This and the following modifications are applied after adapter trimming.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--length",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/trimN",
                    "label": "trimN",
                    "doc": "Trim N's on ends of reads.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--trim-n",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/lengthTag",
                    "label": "lengthTag",
                    "doc": "Search for TAG followed by a decimal number in the description field of the read. Replace the decimal number with the correct length of the trimmed read. For example, use --length-tag 'length=' to correct fields like 'length=123'.",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--length-tag",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/stripSuffix",
                    "label": "stripSuffix",
                    "doc": " Remove this suffix from read names if present. Can be given multiple times.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--strip-suffix",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/prefix",
                    "label": "prefix",
                    "doc": "(-x)  Add this prefix to read names. Use {name} to insert the name of the matching adapter.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--prefix",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/suffix",
                    "label": "suffix",
                    "doc": "(-y)  Add this suffix to read names; can also include {name}",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--suffix",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/zeroCap",
                    "label": "zeroCap",
                    "doc": "(-z) Change negative quality values to zero.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--zero-cap",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/minimumLength",
                    "label": "minimumLength",
                    "doc": "(-m)  Discard reads shorter than LEN. Default: 0",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--minimum-length",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/maximumLength",
                    "label": "maximumLength",
                    "doc": "(-M)  Discard reads longer than LEN. Default: no limit",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--maximum-length",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/maxN",
                    "label": "maxN",
                    "doc": "Discard reads with more than COUNT 'N' bases. If COUNT is a number between 0 and 1, it is interpreted as a fraction of the read length.",
                    "type": [
                        "float",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--max-n",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/discardTrimmed",
                    "label": "discardTrimmed",
                    "doc": "(--discard)  Discard reads that contain an adapter. Use also -O to avoid discarding too many randomly matching reads.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--discard-trimmed",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/discardUntrimmed",
                    "label": "discardUntrimmed",
                    "doc": "(--trimmed-only)  Discard reads that do not contain an adapter.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--discard-untrimmed",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/discardCasava",
                    "label": "discardCasava",
                    "doc": "Discard reads that did not pass CASAVA filtering (header has :Y:).",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--discard-casava",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/quiet",
                    "label": "quiet",
                    "doc": "Print only error messages. Which type of report to print: 'full' or 'minimal'. Default: full",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--quiet",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/compressionLevel",
                    "label": "compressionLevel",
                    "doc": "Use compression level 1 for gzipped output files (faster, but uses more space)",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-Z",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/infoFile",
                    "label": "infoFile",
                    "doc": "Write information about each read and its adapter matches into FILE. See the documentation for the file format.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--info-file",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/restFile",
                    "label": "restFile",
                    "doc": "(-r)  When the adapter matches in the middle of a read, write the rest (after the adapter) to FILE.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--rest-file",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/wildcardFile",
                    "label": "wildcardFile",
                    "doc": "When the adapter has N wildcard bases, write adapter bases matching wildcard positions to FILE. (Inaccurate with indels.)",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--wildcard-file",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/tooShortOutput",
                    "label": "tooShortOutput",
                    "doc": " Write reads that are too short (according to length specified by -m) to FILE. Default: discard reads",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--too-short-output",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/tooLongOutput",
                    "label": "tooLongOutput",
                    "doc": " Write reads that are too long (according to length specified by -M) to FILE. Default: discard reads",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--too-long-output",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/untrimmedOutput",
                    "label": "untrimmedOutput",
                    "doc": " Write reads that do not contain any adapter to FILE. Default: output to same file as trimmed reads",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--untrimmed-output",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/removeMiddle3Adapter",
                    "label": "removeMiddle3Adapter",
                    "doc": "3' adapter to be removed from second read in a pair.",
                    "type": [
                        {
                            "type": "array",
                            "inputBinding": {
                                "prefix": "-A",
                                "separate": true
                            },
                            "items": "string"
                        },
                        "null"
                    ],
                    "inputBinding": {}
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/removeMiddle5Adapter",
                    "label": "removeMiddle5Adapter",
                    "doc": "5' adapter to be removed from second read in a pair.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-G",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/removeMiddleBothAdapter",
                    "label": "removeMiddleBothAdapter",
                    "doc": "5'/3 adapter to be removed from second read in a pair.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-B",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/removeNBasesFromSecondRead",
                    "label": "removeNBasesFromSecondRead",
                    "doc": "Remove LENGTH bases from second read in a pair.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "-U",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/pairAdapters",
                    "label": "pairAdapters",
                    "doc": "Treat adapters given with -a/-A etc. as pairs. Either both or none are removed from each read pair.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--pair-adapters",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/pairFilter",
                    "label": "pairFilter",
                    "doc": "{any,both,first} Which of the reads in a paired-end read have to match the filtering criterion in order for the pair to be filtered. Default: any",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--pair-filter",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/interleaved",
                    "label": "interleaved",
                    "doc": "Read and write interleaved paired-end reads.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--interleaved",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/untrimmedPairedOutput",
                    "label": "untrimmedPairedOutput",
                    "doc": " Write second read in a pair to this FILE when no adapter was found. Use with --untrimmed-output. Default: output to same file as trimmed reads",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--untrimmed-paired-output",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/tooShortPairedOutput",
                    "label": "tooShortPairedOutput",
                    "doc": " Write second read in a pair to this file if pair is too short. Use also --too-short-output.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--too-short-paired-output",
                        "separate": true
                    }
                },
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/tooLongPairedOutput",
                    "label": "tooLongPairedOutput",
                    "doc": " Write second read in a pair to this file if pair is too long. Use also --too-long-output.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--too-long-paired-output",
                        "separate": true
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#cutadapt_2_1.cwl/cutadapt/out",
                    "label": "out",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputBinding": {
                        "glob": "*.fastq.gz",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": "cutadapt",
            "arguments": [],
            "id": "#cutadapt_2_1.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "FastQC",
            "doc": "FastQC is a program designed to spot potential problems in high througput sequencing datasets. It runs a set of analyses on one or more raw sequence files in fastq or bam format and produces a report which summarises the results.\nFastQC will highlight any areas where this library looks unusual and where you should take a closer look. The program is not tied to any specific type of sequencing technique and can be used to look at libraries coming from a large number of different experiment types (Genomic Sequencing, ChIP-Seq, RNA-Seq, BS-Seq etc etc).",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/fastqc:0.11.8--2"
                }
            ],
            "inputs": [
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/reads",
                    "label": "reads",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 5
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/outdir",
                    "label": "outdir",
                    "doc": "(-o) Create all output files in the specified output directory. Please note that this directory must exist as the program will not create it.  If this option is not set then the output file for each sequence file is created in the same directory as the sequence file which was processed.",
                    "type": "string",
                    "default": ".",
                    "inputBinding": {
                        "prefix": "--outdir"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/casava",
                    "label": "casava",
                    "doc": "Files come from raw casava output. Files in the same sample group (differing only by the group number) will be analysed as a set rather than individually. Sequences with the filter flag set in the header will be excluded from the analysis. Files must have the same names given to them by casava (including being gzipped and ending with .gz) otherwise they won't be grouped together correctly.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--casava"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/nano",
                    "label": "nano",
                    "doc": "Files come from naopore sequences and are in fast5 format. In this mode you can pass in directories to process and the program will take in all fast5 files within those directories and produce a single output file from the sequences found in all files.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--nano"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/nofilter",
                    "label": "nofilter",
                    "doc": "If running with --casava then don't remove read flagged by casava as poor quality when performing the QC analysis.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--nofilter"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/extract",
                    "label": "extract",
                    "doc": "If set then the zipped output file will be uncompressed in the same directory after it has been created.  By default this option will be set if fastqc is run in non-interactive mode.",
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "prefix": "--extract"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/java",
                    "label": "java",
                    "doc": "(-j) Provides the full path to the java binary you want to use to launch fastqc. If not supplied then java is assumed to be in your path.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--java"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/noextract",
                    "label": "noextract",
                    "doc": "Do not uncompress the output file after creating it.  You should set this option if you donot wish to uncompress the output when running in non-interactive mode. ",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--noextract"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/nogroup",
                    "label": "nogroup",
                    "doc": "Disable grouping of bases for reads >50bp. All reports will show data for every base in the read. WARNING: Using this option will cause fastqc to crash and burn if you use it on really long reads, and your plots may end up a ridiculous size. You have been warned! ",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--nogroup"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/format",
                    "label": "format",
                    "doc": "(-f) Bypasses the normal sequence file format detection and forces the program to use the specified format.  Valid formats are bam,sam,bam_mapped,sam_mapped and fastq ",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--format"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/threads",
                    "label": "threads",
                    "doc": "(-t) Specifies the number of files which can be processed simultaneously. Each thread will be allocated 250MB of memory so you shouldn't run more threads than your available memory will cope with, and not more than 6 threads on a 32 bit machine",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--threads",
                        "valueFrom": "$([inputs.runtime_cpu, 1, 1].filter(function (inner) { return inner != null })[0])"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/contaminants",
                    "label": "contaminants",
                    "doc": "(-c) Specifies a non-default file which contains the list of contaminants to screen overrepresented sequences against. The file must contain sets of named contaminants in the form name[tab]sequence.  Lines prefixed with a hash will be ignored.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--contaminants"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/adapters",
                    "label": "adapters",
                    "doc": "(-a) Specifies a non-default file which contains the list of adapter sequences which will be explicity searched against the library. The file must contain sets of named adapters in the form name[tab]sequence. Lines prefixed with a hash will be ignored.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--adapters"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/limits",
                    "label": "limits",
                    "doc": "(-l) Specifies a non-default file which contains a set of criteria which will be used to determine the warn/error limits for the various modules.  This file can also be used to selectively  remove some modules from the output all together. The format needs to mirror the default limits.txt file found in the Configuration folder.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--limits"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/kmers",
                    "label": "kmers",
                    "doc": "(-k) Specifies the length of Kmer to look for in the Kmer content module. Specified Kmer length must be between 2 and 10. Default length is 7 if not specified. ",
                    "type": [
                        "int",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--kmers"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/quiet",
                    "label": "quiet",
                    "doc": "(-q) Supress all progress messages on stdout and only report errors.",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--quiet"
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/dir",
                    "label": "dir",
                    "doc": "(-d) Selects a directory to be used for temporary files written when generating report images.Defaults to system temp directory if not specified.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--dir"
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/out",
                    "label": "out",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputBinding": {
                        "glob": "*.zip",
                        "loadContents": false
                    }
                },
                {
                    "id": "#fastqc_v0_11_8.cwl/fastqc/datafile",
                    "label": "datafile",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputBinding": {
                        "glob": "*/fastqc_data.txt",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": "fastqc",
            "arguments": [],
            "id": "#fastqc_v0_11_8.cwl"
        },
        {
            "class": "Workflow",
            "label": "Merge and Mark Duplicates",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/bams",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "secondaryFiles": [
                        ".bai"
                    ]
                },
                {
                    "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/createIndex",
                    "type": "boolean",
                    "default": true
                },
                {
                    "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/maxRecordsInRam",
                    "type": "int",
                    "default": 5000000
                },
                {
                    "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/sampleName",
                    "type": [
                        "string",
                        "null"
                    ]
                },
                {
                    "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles_useThreading",
                    "doc": "Option to create a background thread to encode, compress and write to disk the output file. The threaded version uses about 20% more CPU and decreases runtime by ~20% when writing out a compressed BAM file.",
                    "type": "boolean",
                    "default": true
                },
                {
                    "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles_validationStringency",
                    "doc": "Validation stringency for all SAM files read by this program. Setting stringency to SILENT can improve performance when processing a BAM file in which variable-length data (read, qualities, tags) do not otherwise need to be decoded.The --VALIDATION_STRINGENCY argument is an enumerated type (ValidationStringency), which can have one of the following values: [STRICT, LENIENT, SILENT]",
                    "type": "string",
                    "default": "SILENT"
                }
            ],
            "outputs": [
                {
                    "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/out",
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "outputSource": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/markDuplicates/out"
                }
            ],
            "steps": [
                {
                    "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles",
                    "label": "GATK4: Merge SAM Files",
                    "in": [
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles/bams",
                            "source": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/bams"
                        },
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles/sampleName",
                            "source": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/sampleName"
                        },
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles/useThreading",
                            "source": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles_useThreading"
                        },
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles/createIndex",
                            "source": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/createIndex"
                        },
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles/maxRecordsInRam",
                            "source": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/maxRecordsInRam"
                        },
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles/validationStringency",
                            "source": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles_validationStringency"
                        }
                    ],
                    "run": "#Gatk4MergeSamFiles_4_1_3_0.cwl",
                    "out": [
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles/out"
                        }
                    ]
                },
                {
                    "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/markDuplicates",
                    "label": "GATK4: Mark Duplicates",
                    "in": [
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/markDuplicates/bam",
                            "source": [
                                "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/mergeSamFiles/out"
                            ],
                            "linkMerge": "merge_nested"
                        },
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/markDuplicates/createIndex",
                            "source": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/createIndex"
                        },
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/markDuplicates/maxRecordsInRam",
                            "source": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/maxRecordsInRam"
                        }
                    ],
                    "run": "#Gatk4MarkDuplicates_4_1_3_0.cwl",
                    "out": [
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/markDuplicates/out"
                        },
                        {
                            "id": "#mergeAndMarkBams_4_1_3.cwl/mergeAndMarkBams/markDuplicates/metrics"
                        }
                    ]
                }
            ],
            "id": "#mergeAndMarkBams_4_1_3.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "Performance Summary",
            "doc": "usage: performance_summary.py [-h] --flagstat FLAGSTAT\n                              --collect_insert_metrics COLLECT_INSERT_METRICS\n                              --coverage COVERAGE -o O\n                              [--target_flagstat TARGET_FLAGSTAT]\n                              [--rmdup_flagstat RMDUP_FLAGSTAT] [--genome]\n\nPerformance summary of bam\n\nrequired arguments:\n  --flagstat FLAGSTAT   output of samtools flagstat on bam\n  --collect_insert_metrics COLLECT_INSERT_METRICS\n                        output of CollectInsertMetrics (GATK or Picard) on bam\n  --coverage COVERAGE   output of bedtools coverageBed for targeted bam;\n                        bedtools genomeCoverageBed for whole genome bam\n  -o O                  output summary csv name\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --target_flagstat TARGET_FLAGSTAT\n                        output of samtools flagstat of bam target on target\n                        bed. Only specified for targeted bam\n  --rmdup_flagstat RMDUP_FLAGSTAT\n                        output of samtools flagstat of removed duplicates bam.\n                        File to be used to extract mapping infomation if\n                        specified, instead of the --flagstat file.\n  --genome              calculate statistics for whole genome data.\n                        --target_flagstat must not be speicified\n        ",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "michaelfranklin/pmacutil:0.0.7"
                }
            ],
            "inputs": [
                {
                    "id": "#performanceSummary_0_0_7.cwl/performanceSummary/flagstat",
                    "label": "flagstat",
                    "doc": "output of samtools flagstat on bam",
                    "type": "File",
                    "inputBinding": {
                        "prefix": "--flagstat"
                    }
                },
                {
                    "id": "#performanceSummary_0_0_7.cwl/performanceSummary/collectInsertSizeMetrics",
                    "label": "collectInsertSizeMetrics",
                    "doc": "output of CollectInsertMetrics (GATK or Picard) on bam",
                    "type": "File",
                    "inputBinding": {
                        "prefix": "--collect_insert_metrics"
                    }
                },
                {
                    "id": "#performanceSummary_0_0_7.cwl/performanceSummary/coverage",
                    "label": "coverage",
                    "doc": "output of bedtools coverageBed for targeted bam; bedtools genomeCoverageBed for whole genome bam",
                    "type": "File",
                    "inputBinding": {
                        "prefix": "--coverage"
                    }
                },
                {
                    "id": "#performanceSummary_0_0_7.cwl/performanceSummary/outputPrefix",
                    "label": "outputPrefix",
                    "doc": "prefix of output summary csv",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.csv",
                    "inputBinding": {
                        "prefix": "-o"
                    }
                },
                {
                    "id": "#performanceSummary_0_0_7.cwl/performanceSummary/targetFlagstat",
                    "label": "targetFlagstat",
                    "doc": "output of samtools flagstat of bam target on target bed. Only specified for targeted bam",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--target_flagstat"
                    }
                },
                {
                    "id": "#performanceSummary_0_0_7.cwl/performanceSummary/rmdupFlagstat",
                    "label": "rmdupFlagstat",
                    "doc": "output of samtools flagstat of removed duplicates bam. File to be used to extract mapping infomation if specified, instead of the --flagstat file.",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--rmdup_flagstat"
                    }
                },
                {
                    "id": "#performanceSummary_0_0_7.cwl/performanceSummary/genome",
                    "label": "genome",
                    "doc": "calculate statistics for whole genome data.--target_flagstat must not be speicified",
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--genome"
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#performanceSummary_0_0_7.cwl/performanceSummary/out",
                    "label": "out",
                    "type": "File",
                    "outputBinding": {
                        "glob": "$((inputs.outputPrefix + \".csv\"))",
                        "outputEval": "$((inputs.outputPrefix.basename + \".csv\"))",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": "performance_summary.py",
            "arguments": [],
            "id": "#performanceSummary_0_0_7.cwl"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "id": "#main/reads",
                    "type": {
                        "type": "array",
                        "items": {
                            "type": "array",
                            "items": "File"
                        }
                    }
                },
                {
                    "id": "#main/sample_name",
                    "type": "string"
                },
                {
                    "id": "#main/reference",
                    "type": "File",
                    "secondaryFiles": [
                        ".fai",
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        "^.dict"
                    ]
                },
                {
                    "id": "#main/cutadapt_adapters",
                    "type": [
                        "File",
                        "null"
                    ]
                },
                {
                    "id": "#main/gatk_intervals",
                    "type": {
                        "type": "array",
                        "items": "File"
                    }
                },
                {
                    "id": "#main/snps_dbsnp",
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ]
                },
                {
                    "id": "#main/snps_1000gp",
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ]
                },
                {
                    "id": "#main/known_indels",
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ]
                },
                {
                    "id": "#main/mills_indels",
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ]
                },
                {
                    "id": "#main/align_and_sort_sortsam_tmpDir",
                    "doc": "Undocumented option",
                    "type": [
                        "string",
                        "null"
                    ]
                }
            ],
            "outputs": [
                {
                    "id": "#main/out_bam",
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "outputSource": "#main/merge_and_mark/out"
                },
                {
                    "id": "#main/out_fastqc_reports",
                    "type": {
                        "type": "array",
                        "items": {
                            "type": "array",
                            "items": "File"
                        }
                    },
                    "outputSource": "#main/fastqc/out"
                },
                {
                    "id": "#main/out_performance_summary",
                    "type": "File",
                    "outputSource": "#main/performance_summary/performanceSummaryOut"
                }
            ],
            "steps": [
                {
                    "id": "#main/fastqc",
                    "label": "FastQC",
                    "in": [
                        {
                            "id": "#main/fastqc/reads",
                            "source": "#main/reads"
                        }
                    ],
                    "scatter": [
                        "#main/fastqc/reads"
                    ],
                    "run": "#fastqc_v0_11_8.cwl",
                    "out": [
                        {
                            "id": "#main/fastqc/out"
                        },
                        {
                            "id": "#main/fastqc/datafile"
                        }
                    ]
                },
                {
                    "id": "#main/getfastqc_adapters",
                    "label": "Parse FastQC Adaptors",
                    "in": [
                        {
                            "id": "#main/getfastqc_adapters/fastqc_datafiles",
                            "source": "#main/fastqc/datafile"
                        },
                        {
                            "id": "#main/getfastqc_adapters/cutadapt_adaptors_lookup",
                            "source": "#main/cutadapt_adapters"
                        }
                    ],
                    "scatter": [
                        "#main/getfastqc_adapters/fastqc_datafiles"
                    ],
                    "run": "#ParseFastqcAdaptors_v0_1_0.cwl",
                    "out": [
                        {
                            "id": "#main/getfastqc_adapters/adaptor_sequences"
                        }
                    ]
                },
                {
                    "id": "#main/align_and_sort",
                    "label": "Align and sort reads",
                    "in": [
                        {
                            "id": "#main/align_and_sort/sample_name",
                            "source": "#main/sample_name"
                        },
                        {
                            "id": "#main/align_and_sort/reference",
                            "source": "#main/reference"
                        },
                        {
                            "id": "#main/align_and_sort/fastq",
                            "source": "#main/reads"
                        },
                        {
                            "id": "#main/align_and_sort/cutadapt_adapter",
                            "source": "#main/getfastqc_adapters/adaptor_sequences"
                        },
                        {
                            "id": "#main/align_and_sort/cutadapt_removeMiddle3Adapter",
                            "source": "#main/getfastqc_adapters/adaptor_sequences"
                        },
                        {
                            "id": "#main/align_and_sort/sortsam_tmpDir",
                            "source": "#main/align_and_sort_sortsam_tmpDir"
                        }
                    ],
                    "scatter": [
                        "#main/align_and_sort/fastq",
                        "#main/align_and_sort/cutadapt_adapter",
                        "#main/align_and_sort/cutadapt_removeMiddle3Adapter"
                    ],
                    "scatterMethod": "dotproduct",
                    "run": "#BwaAligner_1_0_0.cwl",
                    "out": [
                        {
                            "id": "#main/align_and_sort/out"
                        }
                    ]
                },
                {
                    "id": "#main/merge_and_mark",
                    "label": "Merge and Mark Duplicates",
                    "in": [
                        {
                            "id": "#main/merge_and_mark/bams",
                            "source": "#main/align_and_sort/out"
                        },
                        {
                            "id": "#main/merge_and_mark/sampleName",
                            "source": "#main/sample_name"
                        }
                    ],
                    "run": "#mergeAndMarkBams_4_1_3.cwl",
                    "out": [
                        {
                            "id": "#main/merge_and_mark/out"
                        }
                    ]
                },
                {
                    "id": "#main/calculate_performancesummary_genomefile",
                    "label": "Generate genome for BedtoolsCoverage",
                    "in": [
                        {
                            "id": "#main/calculate_performancesummary_genomefile/reference",
                            "source": "#main/reference"
                        }
                    ],
                    "run": "#GenerateGenomeFileForBedtoolsCoverage_v0_1_0.cwl",
                    "out": [
                        {
                            "id": "#main/calculate_performancesummary_genomefile/out"
                        }
                    ]
                },
                {
                    "id": "#main/performance_summary",
                    "label": "Performance summary workflow (whole genome)",
                    "in": [
                        {
                            "id": "#main/performance_summary/bam",
                            "source": "#main/merge_and_mark/out"
                        },
                        {
                            "id": "#main/performance_summary/sample_name",
                            "source": "#main/sample_name"
                        },
                        {
                            "id": "#main/performance_summary/genome_file",
                            "source": "#main/calculate_performancesummary_genomefile/out"
                        }
                    ],
                    "run": "#PerformanceSummaryGenome_v0_1_0.cwl",
                    "out": [
                        {
                            "id": "#main/performance_summary/performanceSummaryOut"
                        }
                    ]
                }
            ],
            "id": "#main"
        }
    ],
    "cwlVersion": "v1.0"
}
